mod hpp_out;

use crate::images::args::{Arguments as IMGArguments, ClutAlignment, ColorType};
use clap::Args;
use hpp_out::CppHeader;
use std::path::PathBuf;
use tool_helper::{Error, Input};
use tim_tool::logic::{tim::types::Encoding, project::*};

#[derive(Args)]
pub struct Arguments {
     #[clap(long="hpp", value_parser)]
    hpp_out_path: Option<PathBuf>
}

pub fn run_project(input: Input, args: Arguments, input_path: Option<PathBuf>, output_path: &Option<PathBuf>) -> Result<(), Error> {
    let location_path = if let Some(input_path)  = input_path {input_path} else {PathBuf::from(".")};
    let output_path   = if let Some(output_path) = &output_path {
        let mut output_path = output_path.clone();
        if output_path.is_file() {
            output_path.pop();
        }
        output_path
    } else {PathBuf::from(".")};
    let     project = serde_json::from_reader::<Input, Project>(input).map_err(|error|{Error::from_text(format!("Reading project failed: {}", error))})?;
    let mut hpp_out = CppHeader::new(&args.hpp_out_path)?;

    for job in project.jobs {
        hpp_out.push_job(&job)?;

        let args        = to_arguments(&job)?;
        let output_path = {
            let mut path = output_path.clone();

            path.push(job.name);
            path.set_extension("img");
            path
        };

        let sub_command = crate::SubCommands::SimpleTIM(args);
        crate::run_subcommand(job.settings.compress, Some(Job::create_file_path(job.file_path, &location_path)), Some(output_path), sub_command)?;
    }

    Ok(())
}

fn to_arguments(job: &Job) -> Result<IMGArguments, Error> {
    let (semi_transparent, transparent_palette) = get_transparency(&job.settings.transparency);

    return Ok(IMGArguments {
        color_depth: get_encoding(&job.settings.encoding),
        clut_align:  get_palette_rect(&job.palette_rect)?,
        semi_transparent,
        transparent_palette,    
    });

    fn get_transparency(transparency: &Transparency) -> (bool, bool) {
        match transparency {
            Transparency::None       => (false, false),
            Transparency::FirstColor => (false, true),
            Transparency::PSXSemi    => (true, false),
            Transparency::Both       => (true, true),
        }
    }

    fn get_encoding(encoding: &Encoding) -> ColorType {
        match encoding {
            Encoding::FourBit   => ColorType::Clut4,
            Encoding::EightBit  => ColorType::Clut8,
            Encoding::FullColor => ColorType::Full16,
        }
    }

    fn get_palette_rect(rect: &Option<PaletteRect>) -> Result<ClutAlignment, Error> {
        if let Some(rect) = rect {
            if rect.size.height == 1 {
                return Ok(ClutAlignment::Linear);
            }

            if rect.size.width%16 == 0 {
                return Ok(ClutAlignment::Block);
            }

            return Err(Error::from_text(format!("psxfileconv currently only supports linear or block CLUTs with a width of 16px")));
        }

        Ok(ClutAlignment::None)
    }
}